Attribute VB_Name = "Module1"
Option Explicit

Type TCertificate
  SignersName As String * 256
  Validity  As Byte
End Type

Type TUserId
  Name As String * 256
  Validity     As Byte
  Cert(0 To 15) As TCertificate
End Type

Type TSubKey
  KeyID As String * 18
  Fprint As String * 20
  CreationDate As Long
  ExpireDate As Long
  KeyBits As Long
  KeyAlg As Byte
  PropFlags As Long
End Type

Type TKey
  KeyID As String * 18
  Fingerprint As String * 20
  KeyBits As Long
  CreationDate As Long
  ExpireDate As Long
  PrimaryUserId As Byte ' index into UserID array
  KeyAlg As Byte
  Trust As Byte
  Validity As Byte
  PropFlags As Long
  SubKey(0 To 31) As TSubKey
  'UserID(0 To 31) As TUserId
End Type

'Type TKeyArray
'  Count As Long           ' Number of items in Keys() actually used
'  Keys(170) As TKey       ' Can't total > 32k? 170 * 186 = 31620 (Delphi says 32152)
'End Type

'Type TUserIdArray
'  Count As Long
'  IDs(15) As TUserId
'End Type

'Global KeyRing As TKeyArray

'*****************
' data structure for signature information
Global SIGNED As Boolean
Type TSig_Data
    Status As String
    UserID As String
    KeyID As String
    DateTimeStr As String
    DateTimeInt As Long
    Checked As Boolean
    Verified As Boolean
    KeyValidity As String
    KeyRevoked As Boolean
    KeyDisabled As Boolean
    KeyExpired As Boolean
    KeyIsOnRing As Boolean
End Type
' 1st data structure for key information
Type TKey_Data
    Private As Boolean
    KeyID As String
    UserID As String
    Bits As String
    DateTimeStr As String
    DateTimeInt As String
    Fingerprint As String
    KeyAlgorithm As String
    Trust As String
    Validity As String
End Type

Global KeyArray() As TKey_Data

' One cheesey string-parsing function, coming right up!
' (takes as an argument the tab-delimited string produced by
' decode/decodefile functions & parses it to populate a TSig_Data structure)
Public Function ParseSigData(SigProps As String) As TSig_Data
  Dim pos1 As Integer, pos2 As Integer, sublen As Integer
  Dim Sig As TSig_Data

  If Trim(SigProps) = Chr(0) Then
    SIGNED = False
    Exit Function
  End If

  ' Status - the apparent status of the signature:
  ' SIGNED_NOT      -unsigned
  ' SIGNED_GOOD     -signing key found, data intact
  ' SIGNED_BAD      -data not intact
  ' SIGNED_NO_KEY   -signing key not found, data unverified
'  Global Const SIGNED_GOOD = 0
'  Global Const SIGNED_NOT = 1
'  Global Const SIGNED_BAD = 2
'  Global Const SIGNED_NO_KEY = 3

  pos1 = 1
  pos2 = InStr(1, SigProps, Chr(9))
  sublen = pos2 - 1
  Select Case Left(SigProps, 1)
    Case "0"
    Sig.Status = "SIGNED_GOOD"
    Case "1"
    Sig.Status = "SIGNED_NOT"
    Case "2"
    Sig.Status = "SIGNED_BAD"
    Case "3"
    Sig.Status = "SIGNED_NO_KEY"
    Case Else
    Sig.Status = "SIGNED_NOT"
  End Select
  
  ' if there appears to be a signature
  If Sig.Status <> "" And Not Sig.Status = "SIGNED_NOT" Then
  
  SIGNED = True
  
  ' UserID - primary user id of signing key
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, SigProps, Chr(9))
  sublen = pos2 - pos1
  Sig.UserID = Trim(Mid(SigProps, pos1, sublen))
  
  ' KeyID - key id of signing key
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, SigProps, Chr(9))
  sublen = pos2 - pos1
  Sig.KeyID = Trim(Mid(SigProps, pos1, sublen))
  
  ' DateTimeStr - date & time as a ctime-format string (local time)
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, SigProps, Chr(9))
  sublen = pos2 - pos1
  Sig.DateTimeStr = Trim(Mid(SigProps, pos1, sublen))
  
  ' DateTimeInt - date & time as a ctime-style number (GMT/UTC)
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, SigProps, Chr(9))
  sublen = pos2 - pos1
  Sig.DateTimeInt = Int(Trim(Mid(SigProps, pos1, sublen)))
  
  ' Checked - is signing key available/is message properly formatted?
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, SigProps, Chr(9))
  sublen = pos2 - pos1
  Sig.Checked = Trim(Mid(SigProps, pos1, sublen))
  
  ' Verified - is Checked true and is the data intact?
  ' ( this is the one to check: if this is true, the sig. is good )
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, SigProps, Chr(9))
  sublen = pos2 - pos1
  Sig.Verified = Trim(Mid(SigProps, pos1, sublen))
  
  ' KeyValidity - validity level of signing key:
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, SigProps, Chr(9))
  sublen = pos2 - pos1
  Select Case Trim(Mid(SigProps, pos1, sublen))
  Case "0"
  Sig.KeyValidity = "Unknown"
  Case "1"
  Sig.KeyValidity = "Invalid"
  Case "2"
  Sig.KeyValidity = "Marginal"
  Case "3"
  Sig.KeyValidity = "Complete"
  Case Else
  Sig.KeyValidity = "Uknown"
  End Select

  ' misc. key problems
  
  ' KeyRevoked
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, SigProps, Chr(9))
  sublen = pos2 - pos1
  Sig.KeyRevoked = Trim(Mid(SigProps, pos1, sublen))
  
  ' KeyDisabled
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, SigProps, Chr(9))
  sublen = pos2 - pos1
  Sig.KeyDisabled = Trim(Mid(SigProps, pos1, sublen))
  
  ' KeyExpired
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, SigProps, Chr(9))
  sublen = pos2 - pos1
  Sig.KeyExpired = Trim(Mid(SigProps, pos1))
  End If
  
  ParseSigData = Sig
End Function

Public Function ParseKeyData(KeyProps As String) As TKey_Data
  Dim i As Integer, pos1 As Integer, pos2 As Integer, sublen As Integer, X As Integer
  Dim Key As TKey_Data
  Dim s As String, T As String

  If Len(KeyProps) < 5 Then Exit Function
  
  ' Private?
  pos1 = 1
  pos2 = InStr(1, KeyProps, Chr(9))
  sublen = pos2 - 1
  'If Trim(InStr(pos1, KeyProps, "1")) <> 0 Then
  If Left(KeyProps, 1) = "1" Then
    Key.Private = True
  Else
    Key.Private = False
  End If
  
  ' Key ID
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, KeyProps, Chr(9))
  sublen = pos2 - pos1
  Key.KeyID = Trim(Mid(KeyProps, pos1, sublen))
    
  ' UserID - primary user id of key
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, KeyProps, Chr(9))
  sublen = pos2 - pos1
  Key.UserID = Trim(Mid(KeyProps, pos1, sublen))
  
  ' Key length
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, KeyProps, Chr(9))
  sublen = pos2 - pos1
  Key.Bits = Trim(Mid(KeyProps, pos1, sublen))

  ' DateTimeStr - date & time as a ctime-format string (local time)
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, KeyProps, Chr(9))
  sublen = pos2 - pos1
  Key.DateTimeStr = Trim(Mid(KeyProps, pos1, sublen))
  
  ' DateTimeInt - date & time as a ctime-style number (GMT/UTC)
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, KeyProps, Chr(9))
  sublen = pos2 - pos1
  Key.DateTimeInt = Trim(Mid(KeyProps, pos1, sublen))
  
  ' Fingerprint
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, KeyProps, Chr(9))
  sublen = pos2 - pos1
  Key.Fingerprint = Trim(Mid(KeyProps, pos1, sublen))
  
  ' Key Trust
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, KeyProps, Chr(9))
  sublen = pos2 - pos1
  Select Case Trim(Mid(KeyProps, pos1, sublen))
    Case "0"
    Key.Trust = "Undefined"
    Case "1"
    Key.Trust = "Unknown"
    Case "2"
    Key.Trust = "Never"
    Case "5"
    Key.Trust = "Marginal"
    Case "6"
    Key.Trust = "Complete"
    Case "7"
    Key.Trust = "Ultimate"
    Case Else
    Key.Trust = "Unknown"
    End Select
  
  ' Key Validity
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, KeyProps, Chr(9))
  sublen = pos2 - pos1
  Select Case Trim(Mid(KeyProps, pos1, sublen))
    Case "0"
    Key.Validity = "Unknown"
    Case "1"
    Key.Validity = "Invalid"
    Case "2"
    Key.Validity = "Marginal"
    Case "3"
    Key.Validity = "Complete"
    Case Else
    Key.Validity = "Unknown"
    End Select
      
  ' Public-key algorithm
  pos1 = pos2 + 1
  pos2 = pos2 + 2 'InStr(pos2 + 1, KeyProps, Chr(13))
  sublen = pos2 - pos1
  Select Case Trim(Mid(KeyProps, pos1, sublen))
    Case "0"
    Key.KeyAlgorithm = "Invalid"
    Case "1"
    Key.KeyAlgorithm = "RSA"
    Case "2"
    Key.KeyAlgorithm = "RSAEncryptOnly"
    Case "3"
    Key.KeyAlgorithm = "RSASignOnly"
    Case "4"
    Key.KeyAlgorithm = "ElGamal"
    Case "5"
    Key.KeyAlgorithm = "DSA"
    Case Else
    Key.KeyAlgorithm = "Invalid"
  End Select
     
  ParseKeyData = Key

End Function
' whack the keyring string into smaller strings,
' push data into global array for later use
Public Sub ChopKeyProps(KeyProps As String, Count As Long)
  Dim pos1 As Integer, pos2 As Integer, sublen As Integer
  Dim Tmp(0 To 255)
  Dim i As Integer
        
  ReDim KeyArray(Count) As TKey_Data
  
' KeyProps strings are tab-delimited and end in CRLF
   
  For i = 0 To Count
  ' first, split the string into lines ending in CRLF
    pos1 = pos2 + 1
    pos2 = InStr(pos2 + 1, KeyProps, Chr(13) & Chr(10))
    If pos2 = 0 Then pos2 = Len(KeyProps)
    sublen = pos2 - pos1
    Tmp(i) = Trim(Mid(KeyProps, pos1, sublen))
  Next i

  ' now parse the strings
  For i = 0 To Count - 1
  ' Private?
  pos1 = 1
  pos2 = InStr(1, Tmp(i), Chr(9))
  sublen = pos2 - 1
  'If Trim(Mid(KeyProps, pos1, sublen)) = "Key_Public" Then
'  If Trim(InStr(pos1, Tmp(i), "Key_Public")) <> 0 Then
  If Left(KeyProps, 1) = "1" Then
    KeyArray(i).Private = True
  Else
    KeyArray(i).Private = False
  End If
  
' Key ID
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, Tmp(i), Chr(9))
  sublen = pos2 - pos1
  KeyArray(i).KeyID = Trim(Mid(Tmp(i), pos1, sublen))
    
  ' UserID - primary user id of key
  pos1 = pos2 + 1
  pos2 = InStr(pos2 + 1, Tmp(i), Chr(9))
  'sublen = pos2 - pos1
  sublen = Len(Tmp(i))
  KeyArray(i).UserID = Trim(Mid(Tmp(i), pos1, sublen))
    
 Next i
  
End Sub

Public Function CountCRLF(KeyProps As String) As Integer
  Dim i As Integer, Count As Integer
  Dim v As String
  
  Count = 0
  For i = 1 To Len(KeyProps)
    v = Mid(KeyProps, i, 1)
    If StrComp(v, Chr(13), 0) = 0 Then Count = Count + 1
    'If InStr(1, v, Chr(10)) > 0 Then count = count + 1
  Next i
  CountCRLF = Count
End Function

' NOTES
' When calling a Delphi/ObjectPascal DLL from VB, strings must be declared
' "ByVal" and must be of fixed length. In my experience, they must also be
' manually null-terminated, e.g., MyString = 'privacy' & Chr(0), though only
' one of my sources confirms this. At any rate, this project won't execute
' properly on my computer without the final null (in fact it GPFs violently
' without it).

